// We need a linked list to base our queue on.
import java.util.LinkedList;

// The queue.
public class MyQueue
{
	// Linked list used for managing queue data.
	private LinkedList			mList;

	// Default constructor. Creates an empty stack.
	public					MyQueue()
	{
		// Create the list that this queue implementation will use.
		mList = new LinkedList();
	}

	// Removes all entries from the stack.
	public void				clear()
	{
		mList.clear();
	}

	// Inserts an item at the 'back' of the queue.
	public void				enqueue(DataElement item)
	{
		// Validate.
		if(item == null) return;

		// Add a copy of the item to linked list's end.
		mList.addLast(item.getCopy());
	}

	// Remove an item from the 'front' of the queue (returns removed item).
	// Returns a copy of the item, not the original.
	public DataElement		dequeue()
	{
		// Get a copy of the first item.
		DataElement item = peak();

		// Validate.
		if(item == null) return(null);

		// Now that we have a copy of the item, remove it from the list.
		mList.removeFirst();

		// Return that item.
		return(item);
	}

	// Peaks at item in the 'front' of the queue (does not remove item from the queue).
	// Returns a copy of the item, not the original.
	public DataElement		peak()
	{
		// Check if list contains any entries.
		if(isEmpty()) return(null);

		// Return copy of front element.
		return( ( (DataElement)(mList.getFirst()) ).getCopy());
	}

	// Prints all items in queue.
	public void				printContents()
	{
		// Check if list is empty.
		if(isEmpty())
		{
			System.out.println("Queue is empty");
			return;
		}

		// Iterate through all entries in the list.
		for(int i=0;i<mList.size();++i)
			System.out.println( ( (DataElement)(mList.get(i)) ).toString() );

		// Extra blank lines.
		System.out.println("");
	}

	// Returns true if the queue is empty.
	public boolean			isEmpty() { return(mList.size() <= 0); }
}