// We need a linked list to base our stack on.
import java.util.LinkedList;

// The stack.
public class MyStack
{
	// Linked list used for managing stack data.
	private LinkedList			mList;

	// Default constructor. Creates an empty stack.
	public					MyStack()
	{
		// Create the list that this stack implementation will use.
		mList = new LinkedList();
	}

	// Removes all entries from the stack.
	public void				clear()
	{
		mList.clear();
	}

	// Push an item onto the stack.
	// Puts a copy on the stack, not the original.
	public void				push(DataElement item)
	{
		// Validate.
		if(item == null) return;

		// Add a copy of the item to linked list's end.
		mList.addLast(item.getCopy());
	}

	// Pop an item off of the stack (returns popped item).
	// Returns a copy of the item, not the original.
	public DataElement		pop()
	{
		// Get a copy of the last item.
		DataElement item = peak();

		// Validate.
		if(item == null) return(null);

		// Now that we have a copy of the item, remove it from the list.
		mList.removeLast();

		// Return that item.
		return(item);
	}

	// Peaks at item on top of stack (does not remove item from the stack).
	// Returns a copy of the item, not the original.
	public DataElement		peak()
	{
		// Check if list contains any entries.
		if(isEmpty()) return(null);

		// Return copy of last element.
		return( ( (DataElement)(mList.getLast()) ).getCopy());
	}

	// Prints all items in stack.
	public void				printContents()
	{
		// Check if list is empty.
		if(isEmpty())
		{
			System.out.println("Stack is empty");
			return;
		}

		// Iterate through all entries in the list.
		for(int i=0;i<mList.size();++i)
			System.out.println( ( (DataElement)(mList.get(i)) ).toString() );

		// Extra blank lines.
		System.out.println("");
	}

	// Returns true if the stack is empty.
	public boolean			isEmpty() { return(mList.size() <= 0); }
}