/*******************************************************************************
** Person.java
**
** Dan Strother
** 10/21/2003
** Computer Science 143
**
** Assignment #3
*******************************************************************************/

/**
	Implementation of a class representing a person who possesses both a first
	and last name.
*/
public abstract class Person extends DataElement
{
	/** The person's first name */
	private String			mFirstName;
	/** The person's last name */
	private String			mLastName;

	/**
		Default constructor. Initialize name to something.. default.
	*/
	public					Person()
	{
		mFirstName	= "Mister";
		mLastName	= "Default";
	}

	/**
		Param constructor.
	*/
	public					Person(String first, String last)
	{
		mFirstName	= first;
		mLastName	= last;
	}

	/**
		Copy constructor.
	*/
	public					Person(Person other)
	{
		makeCopy((DataElement)other);
	}

	/** Gets the person's first name. */
	public String			getFirstName()	{ return(mFirstName); }

	/** Gets the person's last name. */
	public String			getLastName()	{ return(mLastName); }

	/** Sets the person's first name */
	public void				setFirstName(String name)	{ mFirstName = name; }

	/** Sets the person's last name. */
	public void				setLastName(String name)	{ mLastName = name; }

    /**
    	Returns true if this person's name matches
        other person's name; otherwise false.
    */
    public boolean			equals(DataElement other)
    {
		// Validate param.
		if(other == null) return(false);

		// Compare first and last names to check for a match.
		if( ( ((Person)other).mFirstName	== mFirstName) &&
			( ((Person)other).mLastName		== mLastName) )
			return(true);

		// Nope - not the same.
		return(false);
	}

    /**
    	Returns < 0 if this person is alphabetically earlier than the other.
    	Returns 0 if they are the same.
    	Returns > 0 if this person is alphabetically later than the other.
    */
    public int				compareTo(DataElement other)
    {
		// Validate.
		if(other == null)
			return(0);

		return(mLastName.compareTo(((Person)other).getLastName()));
	}

    /**
    	This person duplicates passed person.
    */
    public void				makeCopy(DataElement other)
    {
		// Validate param.
		if(other == null) return;

		// Copy intrinsic data.
		mFirstName	= ((Person)other).mFirstName;
		mLastName	= ((Person)other).mLastName;
	}

    /**
		Returns a reference to a copy of this person.
    */
    public abstract DataElement getCopy();

	/**
	Converts element data to a string format for output purposes.
	*/
	public String			toString()
	{
		return(mLastName + ", " + mFirstName);
	}
}